import unittest
from autochat.chat import Autochat
from autochat.model import Message


class MockTool:
    def __init__(self, name):
        self.name = name
        self.call_count = 0

    def __repr__(self):
        return f"MockTool(name={self.name}, call_count={self.call_count})"

    def increment(self):
        self.call_count += 1
        return self.call_count


class TestToolRepr(unittest.TestCase):
    def test_tool_repr_in_last_context(self):
        agent = Autochat(provider="openai")
        mock_tool = MockTool("TestTool")
        class_name = mock_tool.__class__.__name__
        tool_id = agent.add_tool(mock_tool)

        last_context = agent.last_context

        assert (
            f"### {class_name}-{tool_id}\nMockTool(name=TestTool, call_count=0)"
            in last_context
        )

        # Increment the mock tool's call count
        agent.functions[f"MockTool-{tool_id}__increment"]()

        last_context = agent.last_context
        assert (
            f"### {class_name}-{tool_id}\nMockTool(name=TestTool, call_count=1)"
            in last_context
        )

    def test_last_context_in_last_message(self):
        """
        We want to check that the last_context is in the last_message

        """
        agent = Autochat(provider="openai")
        mock_tool = MockTool("TestTool")
        class_name = mock_tool.__class__.__name__
        tool_id = agent.add_tool(mock_tool)

        # Mock the provider's fetch method
        def mock_fetch(**kwargs):
            # Get the last message after prepare_messages is called
            messages = agent.provider.prepare_messages(lambda x: x)
            last_message = messages[-1]

            # Verify the last_context is in the last message's content
            self.assertIn(
                f"### {class_name}-{tool_id}\nMockTool(name=TestTool, call_count=0)",
                last_message.parts[0].content,
            )
            return Message(role="assistant", content="Test response")

        agent.provider.fetch = mock_fetch
        agent.ask("Hello")


if __name__ == "__main__":
    unittest.main()



import os
from autochat.providers.openai import OpenAIProvider
from autochat.providers.anthropic import AnthropicProvider
from autochat.providers.openai_hack import OpenAIProviderHack
from autochat.providers.base_provider import BaseProvider, APIProvider

# from autochat.chat import Autochat


# TODO: should probably exploit default model from provider
def get_provider_and_model(  # TODO: get_provider_and_model ?
    # chat: Autochat, # TODO: make AutochatBase ?
    chat,
    provider_name: str = None,
    model: str = None,
) -> list[str, BaseProvider]:  # TODO: rename
    """
    Returns the correct LLM provider based on a string or env vars.
    """
    if not provider_name:
        provider_name = os.getenv("AUTOCHAT_HOST", "openai")

    if isinstance(provider_name, APIProvider):
        provider_key = provider_name
    elif isinstance(provider_name, str):
        try:
            provider_key = APIProvider(provider_name)
        except ValueError:
            raise ValueError(f"Provider {provider_name} is not a valid provider")
    else:
        raise ValueError(f"Invalid provider: {provider_name}")

    if provider_key == APIProvider.OPENAI:
        if not model:
            model = os.getenv("AUTOCHAT_MODEL", "gpt-4o")
        return OpenAIProvider(chat, model=model), model
    elif provider_key == APIProvider.ANTHROPIC:
        if not model:
            model = os.getenv("AUTOCHAT_MODEL", "claude-3-5-sonnet-20240620")
        return AnthropicProvider(chat, model=model), model
    elif provider_key == APIProvider.OPENAI_HACK:
        if not model:
            model = os.getenv("AUTOCHAT_MODEL", "o1-preview")
        return OpenAIProviderHack(chat, model=model), model
    else:
        raise ValueError(f"Provider {provider_key} is not supported")



"""We want to be able to return an image from a function call.
"""

import pytest
from autochat import Autochat, APIProvider
from PIL import Image


def read_image():
    return Image.open("tests/images/mileage.jpg")


@pytest.mark.vcr
def test_anthropic_callback_image():
    agent = Autochat(
        provider=APIProvider.ANTHROPIC,
    )
    agent.add_function(read_image)

    user_query = "Hello, read the image and explain it"
    messages = []
    for message in agent.run_conversation(user_query):
        print(message.to_markdown())  # For debugging
        messages.append(message)


@pytest.mark.vcr
def test_openai_callback_image():
    agent = Autochat(
        provider=APIProvider.OPENAI,
    )
    agent.add_function(read_image)

    user_query = "Hello, read the image and explain it"
    messages = []
    for message in agent.run_conversation(user_query):
        print(message.to_markdown())  # For debugging
        messages.append(message)
